#
# Copyright and User License
# ~~~~~~~~~~~~~~~~~~~~~~~~~~
# Copyright European Organization for Nuclear Research (CERN)
# All rights reserved
#
# DISCLAIMER
# ~~~~~~~~~~
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
# NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY, OF
# SATISFACTORY QUALITY, AND FITNESS FOR A PARTICULAR PURPOSE
# OR USE ARE DISCLAIMED. THE COPYRIGHT HOLDERS AND THE
# AUTHORS MAKE NO REPRESENTATION THAT THE SOFTWARE AND
# MODIFICATIONS THEREOF, WILL NOT INFRINGE ANY PATENT,
# COPYRIGHT, TRADE SECRET OR OTHER PROPRIETARY RIGHT.
#
# LIMITATION OF LIABILITY
# ~~~~~~~~~~~~~~~~~~~~~~~
# THE COPYRIGHT HOLDERS AND THE AUTHORS SHALL HAVE NO
# LIABILITY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL,
# CONSEQUENTIAL, EXEMPLARY, OR PUNITIVE DAMAGES OF ANY
# CHARACTER INCLUDING, WITHOUT LIMITATION, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES, LOSS OF USE, DATA OR PROFITS,
# OR BUSINESS INTERRUPTION, HOWEVER CAUSED AND ON ANY THEORY
# OF CONTRACT, WARRANTY, TORT (INCLUDING NEGLIGENCE), PRODUCT
# LIABILITY OR OTHERWISE, ARISING IN ANY WAY OUT OF THE USE OF
# THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
# DAMAGES.
#
# Author:	Vasilis.Vlachoudis@cern.ch
# Date:	07-Apr-2016

# ******************************
# Default variables
# ******************************
SYSTEM ?= UNIX
THREAD ?= yes
MEM    ?= no

# ******************************
# File extensions
# ******************************
UNAME := $(shell uname -s)
ifeq ($(UNAME),Darwin)
	SYSTEM = $(UNAME)
	SO     = so
	OBJ    = o
	EXE    =
else ifneq (,$(findstring CYGWIN,$(UNAME)))
	SYSTEM = CYGWIN
	SO     = dll
	OBJ    = o
	EXE    = .exe
else
	SO     = so
	OBJ    = o
	EXE    =
	OSNAME := $(shell uname -o)
endif

# ******************************
# Compilers
# ******************************
ifeq ($(COMP),clang)
	CXX  := clang++
else ifeq ($(COMP),gcc)
	CXX  := g++
else ifeq ($(COMP),intel)
	CXX  := icc
else
	ifeq ($(COMP),)
		CXX := g++
	else
		CXX := $(COMP)
	endif
	ifeq ($(shell $(CXX) -v 2>&1 | grep -c "clang version"), 1)
		COMP := clang
	else
		COMP := gcc
	endif
endif

# ******************************
# Architecture
# ******************************
TARGET ?= $(shell uname -m)

ifeq ($(TARGET),i386)
	ARCH      = -m32

else ifeq ($(TARGET),i586)
	ARCH      = -m32

else ifeq ($(TARGET),i686)
	ARCH      = -m32

else ifeq ($(TARGET),x86_64)
	ARCH      = -m64

else ifeq ($(TARGET),amd64)
	ARCH      = -m64

else ifeq ($(TARGET),armv7l)
#	ARCH      = -m64

else ifeq ($(TARGET),arm64)
	ARCH      = -m64

else ifeq ($(TARGET),aarch64)
	ARCH      = -m64

else ifeq ($(TARGET),android)
	ANDNDK    = /opt/android-ndk
	ANDOS     = 21
	ANDVER    = 4.8
	ANDARCH   = arm
	#ANDBIT    = linux-x86
	#ANDBIT    = linux-x86_64
	#ANDBIN   := $(ANDNDK)/toolchains/$(ANDARCH)-linux-androideabi-$(ANDVER)/prebuilt/$(ANDBIT)/bin
	#ANDBIN    = $(shell ls -d $(ANDNDK)/toolchains/$(ANDARCH)-linux-androideabi-$(ANDVER)/prebuilt/*)/bin
	ANDBIN    = $(shell ls -d $(ANDNDK)/bin/$(ANDARCH)-linux-androideabi)/bin
	ANDROOT  := $(ANDNDK)/platforms/android-$(ANDOS)/arch-$(ANDARCH)
	ANDARM   := $(ANDBIN)/$(ANDARCH)-linux-androideabi
	ANDSTL   := $(ANDNDK)/sources/cxx-stl/stlport
	ARCH      = --sysroot=$(ANDROOT)
	CXX      := $(ANDARM)-c++
	PY       := no
	X11      := no
	SYSTEM   := ANDROID
	THREAD   := no
else
	ERROR := $(error TARGET=$(TARGET) architecture not found)
endif
