////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// F A R M     -  Flair advanced rendering module
//
// Copyright European Organization for Nuclear Research( CERN )
// All rights reserved
//
// Authors: Christian.Theis@cern.ch
//			    Vasilis.Vlachoudis@cern.ch
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#ifndef SPEC_DISPERSIVE_TRANSMISSION_H
#define SPEC_DISPERSIVE_TRANSMISSION_H

#include "bxdfbase.h"
#include "bxdfspectransmission.h"

class CFresnelDiel;

class CBxDFDispersiveSpecTransmission : public CBxDFSpecTransmission
{
public:
  CBxDFDispersiveSpecTransmission( double DispersionPower, unsigned int TotalNrOfSamples, const Color& TransmissionColor = Color( 1.0, 1.0, 1.0 ) );
  virtual ~CBxDFDispersiveSpecTransmission( void ) {};

  // f( Out, In) evaluates the BxDF for the in & outgoing pair of rays
  virtual Color f( const Ray& Out, const Ray& In, const Vector& Normal, const Material& RenderMat, const Color& LightColor = Color( 1.0, 1.0, 1.0 ), double Power = 1.0, float Shade = 1.0f ) override {
    UNUSED( Out ); UNUSED( In ); UNUSED( Normal ); UNUSED( RenderMat ); UNUSED( LightColor ); UNUSED( Power ); UNUSED( Shade );
    return Color::Black;
  }

  // Sample_f samples an incoming ray matching to the outgoing ray + its Pdf. Furthermore it evaluates the
  // BxDF according to In & Out and returns the color. This is used for reflection, refraction etc. which will
  // continue recursively.
  virtual Color Sample_f( const Ray& Out, Ray& In, float& Pdf, const Vector& Normal, const Material& RenderMat, double* pRandNr, BxDFType Type, const Color& LightColor = Color( 1.0, 1.0, 1.0 ), double Power = 1.0, float Shade = 1.0f ) override;
	virtual double GetPDF( const Vector& Out, const Vector& In, const Vector& Normal ) const override {
		UNUSED( In ); UNUSED( Out ); UNUSED( Normal );
		return 1.0;
	};

  void          SetDispersionPower( double Val ) { m_DispersionPower = Val; };
  double        GetDispersionPower() const { return m_DispersionPower; };

  void          SetCurSampleIndex( unsigned int Val ) { m_CurSampleIndex = Val; };
  unsigned int  GetCurSampleIndex() const { return m_CurSampleIndex; };

  void          SetTotalNrOfSamples( unsigned int Val ) { m_TotalNrOfSamples = Val; };
  unsigned int  GetTotalNrOfSamples() const { return m_TotalNrOfSamples; };

protected:
  virtual Color Sample_f_Transmission( const Ray& Out, Ray& In, float& Pdf, const Vector& Normal, const Material& RenderMat, double* pRandNr, const Color& LightColor ) override;

protected:
  double        m_DispersionPower;
  double        m_CauchyA;
  double        m_CauchyB;
  unsigned int  m_CurSampleIndex;
  unsigned int  m_TotalNrOfSamples;
};

#endif
// vim:ts=2
