////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// F A R M     -  Flair advanced rendering module
//
// Copyright European Organization for Nuclear Research( CERN )
// All rights reserved
//
// Authors: Christian.Theis@cern.ch
//			    Vasilis.Vlachoudis@cern.ch
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "bxdftranslucent.h"
#include "bxdffresnel.h"

CBxDFTranslucent::CBxDFTranslucent() : CBxDFSpecTransmission( 0.0, Color( 0.8, 0.8, 0.8 ) )
{
}

CBxDFTranslucent::~CBxDFTranslucent(void)
{
}


////////////////////////////////////////////////////////////////////////////////////////////////////////////

Color CBxDFTranslucent::Sample_f( const Ray& Out, Ray& In, float& Pdf, const Vector& Normal, const Material& RenderMat, double* pRandNr, BxDFType Type, const Color& LightColor /*= Color( 1.0, 1.0, 1.0 )*/, double Power /*= 1.0*/, float Shade /*= 1.0 */ )

////////////////////////////////////////////////////////////////////////////////////////////////////////////
{
	UNUSED( Power );

	if ( Type == CBxDFBase::BSDF_REFLECTION )
		return Sample_f_Reflection( Out, In, Pdf, Normal, RenderMat, pRandNr, LightColor );
	else if ( Type == CBxDFBase::BSDF_TRANSMISSION ) {
		return Sample_f_Transmission( Out, In, Pdf, Normal, RenderMat, pRandNr, LightColor );
	}
	else
		return CBxDFBase::Sample_f( Out, In, Pdf, Normal, RenderMat, pRandNr, Type, LightColor, Shade );

	return Color::Black;
}


////////////////////////////////////////////////////////////////////////////////////////////////////////////

Color CBxDFTranslucent::Sample_f_Transmission( const Ray& Out, Ray& In, float& Pdf, const Vector& Normal, const Material& RenderMat, double* pRandNr, const Color& LightColor )

////////////////////////////////////////////////////////////////////////////////////////////////////////////
{
	auto RetCol = CBxDFSpecTransmission::Sample_f_Transmission( Out, In, Pdf, Normal, RenderMat, pRandNr, LightColor );

	Vector distortedDir = CBaseSampler::uniformSampleCone( In.dir(), std::min( std::max( RenderMat.fuzz() / 3.0, 0.03), 1.0 ), pRandNr[0], pRandNr[1] );
	In.segment().dir = distortedDir;

	return RetCol;

}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

Color CBxDFTranslucent::Sample_f_Reflection( const Ray& Out, Ray& In, float& Pdf, const Vector& Normal, const Material& RenderMat, double* pRandNr, const Color& LightColor )

//  [12/14/2023 theis]
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
{
	auto Rho = m_TransmissionColor * CBxDFSpecTransmission::Sample_f_Reflection( Out, In, Pdf, Normal, RenderMat, pRandNr, LightColor );

	// distort
	Vector T = CBaseSampler::uniformSampleCone( In.dir(), std::min( 1.0, std::max( RenderMat.fuzz() / 3.0, 0.03)), pRandNr[0], pRandNr[1] );
	In.segment().dir = T;

	Vector HalfVec( -Out.dir() + In.dir() );
	HalfVec.normalize();
	double DotInHalf = In.dir().dot( HalfVec );
	if ( DotInHalf > 0.0 )
		return LightColor * Color( CFresnelHelpers::SchlickFresnel( Rho.red(), DotInHalf ), CFresnelHelpers::SchlickFresnel( Rho.green(), DotInHalf ), CFresnelHelpers::SchlickFresnel( Rho.blue(), DotInHalf ) );
		
	return Color::Black;
}

// vim:ts=2
