////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// F A R M     -  Flair advanced rendering module
//
// Copyright European Organization for Nuclear Research( CERN )
// All rights reserved
//
// Authors: Christian.Theis@cern.ch
//			    Vasilis.Vlachoudis@cern.ch
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#ifndef BRUSHED_METAL_H
#define BRUSHED_METAL_H

#include "physmatinterface.h"
#include "bxdfbase.h"
#include "bxdflafortune.h"
#include "baseintegrator.h"
#include "noisetexture.h"


class CBxDFLaFortune;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// standard brushed metal with noise texture
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

class CMatBrushedMetal : public CPhysMatInterface
{
public:
  CMatBrushedMetal( CBaseIntegrator* pInt, bool bNoiseTex = false, double BumpScaling = 1 );
  virtual ~CMatBrushedMetal(void);

	// f( Out, In) evaluates the BxDF for the in & outgoing pair of rays
	// Out = ray coming FROM the camera, In = ray going towards the light
	virtual Color f( const Ray& Out, const Ray& In, const Vector& Normal, const Material& RenderMat, const Color& LightColor, double Power, float Shade = 1.0f ) override;

	// By default only diffuse sampling is performed. This function needs to be overwritten by the actual physical materials that
	// have reflective and probably translucent components!
	virtual Color Sample_f( const Ray& Out, Ray& In, float& Pdf, const Vector& Normal, const Material& RenderMat, double* pRandNr, CBxDFBase::BxDFType Type, const Color& LightColor = Color( 1.0, 1.0, 1.0 ), double Power = 1.0, float Shade = 1.0f ) override;

	// override shading normal & geometry modififiers
	virtual const Vector& GetShadingNormal( const Ray* pCurRay ) override;
	virtual void ModifyShadingGeometry( Ray* pCurRay, GeometryEngine* pEngine ) override;

protected:
  CBxDFLaFortune*						m_pBxDFDiffuse;

	bool											m_bUseNoise;
	double										m_BumpScaling;
};

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// brushed metal with normal displacement maps
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class CMatBrushedMetalTex : public CMatBrushedMetal
{
public:
	CMatBrushedMetalTex( CBaseIntegrator* pInt, const std::string& TexFileName, double TexDimU, double TexDimV, double BumpScaling = 1 ) : CMatBrushedMetal( pInt, false, BumpScaling )
	{
		m_TexFileName = TexFileName;
		m_TexDimU = TexDimU;
		m_TexDimV = TexDimV;

		//InitTextures();
	};

	virtual ~CMatBrushedMetalTex() = default;

	////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	// override shading normal & geometry modififiers
	virtual const Vector& GetShadingNormal( const Ray* pCurRay ) override;
	virtual void ModifyShadingGeometry( Ray* pCurRay, GeometryEngine* pEngine ) override;

	////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	// override color by blending the base color with the image texture
	Color GetHitColor( const Ray* pRay ) override
	{
		return MixBaseWithTexColor( pRay, 0.9 );
	}

};


#endif
// vim:ts=2
