////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// F A R M     -  Flair advanced rendering module
//
// Copyright European Organization for Nuclear Research( CERN )
// All rights reserved
//
// Authors: Christian.Theis@cern.ch
//			    Vasilis.Vlachoudis@cern.ch
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "matfelt.h"

CMatFelt::CMatFelt( CBaseIntegrator* pInt ) : CPhysMatInterface( pInt, static_cast<CBxDFBase::BxDFType>( CBxDFBase::BSDF_DIFFUSE | CBxDFBase::BSDF_REFLECTION ), false /* false = no MC sampling required */ )
{
  static Color Diffuse( 0.025865,  0.025865,  0.025865 );
	static float xy0[3] =     { -0.304075f, -0.304075f, -0.304075f};
	static float z0[3] =      { -0.065992f, -0.065992f, -0.065992f};
	static float e0[3] =      {  3.047892f,  3.047892f,  3.047892f};
	static float xy1[3] =     { -0.749561f, -0.749561f, -0.749561f};
	static float z1[3] =      { -1.167929f, -1.167929f, -1.167929f};
	static float e1[3] =      {  6.931827f,  6.931827f,  6.931827f};
	static float xy2[3] =     {  1.004921f,  1.004921f,  1.004921f};
	static float z2[3] =      { -0.205529f, -0.205529f, -0.205529f};
	static float e2[3] =      { 94.117332f, 94.117332f, 94.117332f};

  static Color XY[3] = { Color( xy0[0], xy0[1], xy0[2]), Color( xy1[0], xy1[1], xy1[2]), Color( xy2[0], xy2[1], xy2[2]) };
  static Color  Z[3] = { Color( z0[0], z0[1], z0[2]), Color( z1[0], z1[1], z1[2]), Color( z2[0], z2[1], z2[2]) };
  static Color  E[3] = { Color( e0[0], e0[1], e0[2]), Color( e1[0], e1[1], e1[2]), Color( e2[0], e2[1], e2[2]) };

  m_pBxDFDiffuse = new CBxDFLaFortune( Diffuse, 3, XY, XY, Z, E, static_cast<CBxDFBase::BxDFType>( CBxDFBase::BSDF_DIFFUSE | CBxDFBase::BSDF_REFLECTION ) );
}

CMatFelt::~CMatFelt(void)
{
  delete m_pBxDFDiffuse;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////

Color CMatFelt::f( const Ray& Out, const Ray& In, const Vector& Normal, const Material& RenderMat, const Color& LightColor, double Power, float Shade /*= 1.0*/ )

// f( Out, In) evaluates the BxDF for the in & outgoing pair of rays
////////////////////////////////////////////////////////////////////////////////////////////////////////////
{
  UNUSED( Power );
  return m_pBxDFDiffuse->f( Out, In, Normal, RenderMat, LightColor, Shade );
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////

Color CMatFelt::Sample_f( const Ray& Out, Ray& In, float& Pdf, const Vector& Normal, const Material& RenderMat, double* pRandNr, CBxDFBase::BxDFType Type, const Color& LightColor /*= Color( 1.0, 1.0, 1.0 )*/, double Power /*= 1.0*/, float Shade /*= 1.0*/ )

// Sample_f samples an incoming ray matching to the outgoing ray + its Pdf. Furthermore it evaluates the
// BxDF according to In & Out and returns the color. This is used for reflection, refraction etc. which will
// continue recursively.
////////////////////////////////////////////////////////////////////////////////////////////////////////////
{
  UNUSED( Power );

  if( Type == CBxDFBase::BSDF_REFLECTION || Type == CBxDFBase::BSDF_DIFFUSE )
    return m_pBxDFDiffuse->Sample_f( Out, In, Pdf, Normal, RenderMat, pRandNr, Type, LightColor, Shade);

  return Color::Black;
}
// vim:ts=2
