/*
 * Copyright European Organization for Nuclear Research (CERN)
 * All rights reserved
 *
 * Author:	Vasilis.Vlachoudis@cern.ch
 * Date:	04-Feb-2010
 */

#include "xyzbody.h"

#include "cylbody.h"
#include "spherebody.h"

using std::cout;
using std::endl;

const double PSMALL = 1e-12;
const double PHUGE  = 1e12;

// ================================= GXYZBody ==================================
/** set */
void GXYZBody::set(int n, double* what)
{
	round0(what, n, SMALL);

	nc = n/2 + (n&1);

	// copy the points
	for (int i=0; i<nc; i++) {
		c[i].z = what[2*i];
		c[i].r = what[2*i+1];
	}

	// make it three points if only one or two are given.
	for (int i=nc; i<3; i++) {
		c[i].z = c[i-1].z;
		c[i].r = c[i-1].r;
		cout << "copy=" << i << endl;
	}
} // set

/** get */
int GXYZBody::get(double* what) const
{
	for (int i=0; i<nc; i++) {
		what[2*i]   = c[i].z;
		what[2*i+1] = c[i].r;
	}
	return nc*2;
} // get

///** position
// * @param x,y,z	new position to set
// */
//void GXYZBody::position(const Point& r)
//{
//	GBody::position(r);
//
//	switch (type()) {
//		case Type::YZP:
//		case Type::PX:
//			P.y = P.z = 0.0;
//			break;
//
//		case Type::XZP:
//		case Type::PY:
//			P.x = P.z = 0.0;
//			break;
//
//		case Type::XYP:
//		case Type::PZ:
//			P.x = P.y = 0.0;
//			break;
//
//		default: NOP;
//	}
//	Po = P + zlen*Z;
//} // position
//
///** move body item to location
// * @param item	item to move as returned by closest
// * @param r	position vector
// * @param w	vertical axis of the viewport
// * WARNING: the w is used for the rotation (20)
// *          and should be either viewport w or -w such as
// *                    ((r-p)^w)*Z > 0
// *          The reason is that the system below gives two solutions
// *          depending on the side that the user start to drag the extremity
// *          and the program has to remember the side
// */
//void GXYZBody::move(int item, const Point& r, const Vector& w)
//{
//	resetBBox();
//	Point T = _hasMatrix? _invMatrix*r : r;
//	T -= SP;
//	if (item==NodeId_axisR) {
//		// Rotate plane using r as a Second node lying on the plane
//		// Calculate new normal
//		// Vector w = SZ ^ T;
//		// Z = T ^ w;
//
//		// We have to rotate the plane around w of the viewport
//		// keeping the dot(W,N')=const
//		// Equations to fulfill
//		// 1. N'.r - N'.p = 0 <=> N'*T = 0
//		// 2. W.N' = W.N
//		// 3. |N'| = |N|
//
//		Point&  d = T;
//		Vector dw = d ^ w;
//		double  c = SZ * w;
//
//		double E = d.x*dw.z;
//		if (!Eq0(E,SMALL)) {
//			double A = c*d.x*d.y;
//			double B = d.y*dw.y + d.z*dw.z;
//			double C = c*d.x*d.x;
//			double D = d.x*dw.y;
//
//			double AA = B*B + D*D + E*E;
//			double BB = 2.0 * (A*B + C*D);
//			double CC = A*A + C*C - E*E;
//
//			if (Eq0(AA,SMALL)) return;
//			double x1, x2;
//			int n = quadratic(BB/AA, CC/AA, &x1, &x2, SMALL);
//			if (n==0) return;
//			Z.z = x1;
//			Z.y =  (C + D*Z.z) / E;
//			Z.x = -(A + B*Z.z) / E;
//
//			if (Sign(dw * Z)*_sign20 < 0) {
//				Z.z = x2;
//				Z.y =  (C + D*Z.z) / E;
//				Z.x = -(A + B*Z.z) / E;
//			}
//		} else {
//			// rotate x->y->z->x
//			E = d.y*dw.x;
//			if (!Eq0(E,SMALL)) {
//				double A = c*d.y*d.z;
//				double B = d.z*dw.z + d.x*dw.x;
//				double C = c*d.y*d.y;
//				double D = d.y*dw.z;
//
//				double AA = B*B + D*D + E*E;
//				double BB = 2.0 * (A*B + C*D);
//				double CC = A*A + C*C - E*E;
//
//				if (Eq0(AA,SMALL)) return;
//				double x1, x2;
//				int n = quadratic(BB/AA, CC/AA, &x1, &x2, SMALL);
//				if (n==0) return;
//				Z.x = x1;
//				Z.z =  (C + D*Z.x) / E;
//				Z.y = -(A + B*Z.x) / E;
//
//				if (Sign(dw * Z)*_sign20 < 0) {
//					Z.x = x2;
//					Z.z =  (C + D*Z.x) / E;
//					Z.y = -(A + B*Z.x) / E;
//				}
//			} else {
//				// rotate x->z, y->x, z->y
//				E = d.z*dw.y;
//				if (Eq0(E,SMALL)) return;
//				double A = c*d.z*d.x;
//				double B = d.x*dw.x + d.y*dw.y;
//				double C = c*d.z*d.z;
//				double D = d.z*dw.x;
//
//				double AA = B*B + D*D + E*E;
//				double BB = 2.0 * (A*B + C*D);
//				double CC = A*A + C*C - E*E;
//
//				if (Eq0(AA,SMALL)) return;
//				double x1, x2;
//				int n = quadratic(BB/AA, CC/AA, &x1, &x2, SMALL);
//				if (n==0) return;
//				Z.y = x1;
//				Z.x =  (C + D*Z.y) / E;
//				Z.z = -(A + B*Z.y) / E;
//
//				if (Sign(dw * Z)*_sign20 < 0) {
//					Z.y = x2;
//					Z.x =  (C + D*Z.y) / E;
//					Z.z = -(A + B*Z.y) / E;
//				}
//			}
//		}
//		assert(Eq(Z*w,c,SMALL));
//
//		Z.normalize();
//		try {
//			findXYZ();
//		} catch (std::string& e) {
//			/* do nothing) */
//		}
//	} else
//	if (item==NodeId_Pos)
//		P = SP + (T*SZ)*SZ;
//	else
//		GBody::move(item, r, w);
//	checkType();
//} // move
//
///** rotate body around an axis */
//void GXYZBody::rotate(const double angle, const Vector& axis)
//{
//	GBody::rotate(angle,axis);
//	checkType();
//} // rotate

/** createQuads */
void GXYZBody::createQuads()
{
	int ks=-1;		// shape type
	int i1,i2,i3;
	double a[4][4];		// quadratic matrix
	const char* hs[] = {
			"plane",			// 0
			"cylinder",			// 1
			"one-sheet cone",		// 2
			"sphere",			// 3
			"paraboloid",			// 4
			"hyperboloid of one sheet",	// 5
			"hyperboloid of 2 sheets",	// 6
			"ellipsoid",			// 7
			"two-sheet cone"		// 8
		};

	GBody::createQuads();

#if 0
	if (nc == 1) {
		// convert a point-defined plane to a p-plane.;
		double tpp[5];
		tpp[4]=0;
		for (int i=1; i<=3; i++) {
			int j = i%3+1;
			int k = 6-i-j;
			tpp[i] = scf[j]*(scf[k+3]-scf[k+6])+scf[j+3]*
				(scf[k+6]-scf[k])+scf[j+6]*(scf[k]-scf[k+3]);
			tpp[4] = tpp[4]+scf[i]*(scf[j+3]*scf[k+6]-
				scf[j+6]*scf[k+3]);
		}

		// check for the points being in line.  put the results in scf.;
		double xm=0;
		for (int i=1; i<=4; i++) {
			if (xm == 0. and tpp[5-i]!=0.)
				xm=1./tpp[5-i];
			scf[5-i] = tpp[5-i]*xm;
		}
		if (xm == 0.)
			printf("cannot convert surface %d to a plane.\n",icn);
		return ks;
	}
#endif
	cout << endl;
	for (int i=0; i<3; i++)
		printf("c[%d] x=%g r=%g\n",i,c[i].z,c[i].r);

	// set up to detect special cases.
	double t, t1, t2;
	t = sqrt(Max(Sq(c[0].z-c[1].z) + Sq(c[0].r-c[1].r),
		     Sq(c[1].z-c[2].z) + Sq(c[1].r-c[2].r),
		     Sq(c[2].z-c[0].z) + Sq(c[2].r-c[0].r)));
	t1 = Min(c[0].z, c[1].z, c[2].z);
	t2 = Max(c[0].z, c[1].z, c[2].z);

	P = Vector::O;
	xlen = ylen = zlen = 0.0;
	int axis;
	switch (type()) {
		case Type::X:
			Z = Vector::Xo;
			axis = 0;
			break;
		case Type::Y:
			Z = Vector::Yo;
			axis = 1;
			break;
		case Type::Z:
			Z = Vector::Zo;
			axis = 2;
			break;
		default:
			assert(false);
			axis = 0;
	}
	findXYZ();

	cout << "t=" << t << endl;
	cout << "t1=" << t1 << endl;
	cout << "t2=" << t2 << endl;

	// plane if it is narrow or cylinder if it is horizontal.
	if (Max(Abs(c[0].r-c[1].r),
		Abs(c[1].r-c[2].r),
		Abs(c[2].r-c[0].r)) <= PSMALL*t) ks = 1;
	if (t2-t1 <= PSMALL*t) ks = 0;
	if (ks==1) {
		//m1c = px/py/pz
		xlen = ylen = (c[0].r + c[1].r + c[2].r)/3;
		switch (type()) {
			case Type::X:
				addQuad(Quad(Quad::Type::CYLINDER_XO, xlen));
				break;
			case Type::Y:
				addQuad(Quad(Quad::Type::CYLINDER_YO, xlen));
				break;
			case Type::Z:
				addQuad(Quad(Quad::Type::CYLINDER_ZO, xlen));
				break;
			default: assert(false);
		}
		cout << "Adding: " << hs[ks] << " with radius=" << xlen << endl;
		return;
	} else
	if (ks==0) {
		// CX/CY/CZ
		double pos = (c[0].z + c[1].z + c[2].z)/3;
		switch (type()) {
			case Type::X:
				P.set(pos,0,0);
				xlen = 1.0;
				addQuad(Quad(Quad::Type::PLANE_X, pos));
				break;
			case Type::Y:
				P.set(0,pos,0);
				xlen = 1.0;
				addQuad(Quad(Quad::Type::PLANE_Y, pos));
				break;
			case Type::Z:
				P.set(0,0,pos);
				addQuad(Quad(Quad::Type::PLANE_Z, pos));
				break;
			default: assert(false);
		}
		cout << "Adding: " << hs[ks] << " at pos=" << P << endl;
		return;
	}

	// one-sheet cone if it is a straight line.
	if (Abs((c[1].z-c[0].z) * (c[2].r-c[1].r)
	       -(c[1].r-c[0].r) * (c[2].z-c[1].z))*PHUGE <= t*t) {
	        // KX KY KZ
		ks = 2;
		t1 = c[0].z + c[1].z + c[2].z;
		t2 = c[0].r + c[1].r + c[2].r;
		double k = (3.*(c[0].z*c[0].r + c[1].z*c[1].r + c[2].z*c[2].r)- t1*t2)
			/ (3.*(Sq(c[0].z) + Sq(c[1].z) + Sq(c[2].z)) - t1*t1);
		double slope = Sq(k);
		double pos = (t1-t2/k)/3.;
		switch (type()) {
			case Type::X:
				P.set(pos,0,0);
				addQuad(Quad(Quad::Type::CONE_X, P, slope));
				addQuad(Quad(Quad::Type::PLANE_X, pos));
				if (k >  SMALL) _Q[1].negate();
				break;
			case Type::Y:
				P.set(0,pos,0);
				addQuad(Quad(Quad::Type::CONE_Y, P, slope));
				addQuad(Quad(Quad::Type::PLANE_Y, pos));
				if (k >  SMALL) _Q[1].negate();
				break;
			case Type::Z:
				P.set(0,0,pos);
				addQuad(Quad(Quad::Type::CONE_Z, P, slope));
				addQuad(Quad(Quad::Type::PLANE_Z, pos));
				if (k >  SMALL) _Q[1].negate();
				break;
			default: assert(false);
		}
		cout << "Adding: " << hs[ks] << " at ="
			<< " pos=" << P
			<< " slope=" << slope
			<< " k=" << k << endl;
		return;
	}

	// check for a plane of two sheets.
	if (!(Abs(c[0].r-c[1].r) <= PHUGE*Abs(c[0].z-c[1].z) and
	      Abs(c[1].r-c[2].r) <= PHUGE*Abs(c[1].z-c[2].z) and
	      Abs(c[2].r-c[0].r) <= PHUGE*Abs(c[2].z-c[0].z))) {
		// error
		printf("Error:, surface would be two parallel planes.\n");
		return;
	}

	// find coefficients of symmetric gq equation by elimination.
	// r**2 + a[3][0]*x**2 + a[3][1]*x + a[3][2] = 0
	i1 = 0;
	for (int i=0; i<3; i++) {
		a[i][0] = Sq(c[i].z);
		if (a[i][0] > a[i1][0]) i1=i;
		a[i][1] = c[i].z;
		a[i][2] = 1.;
		a[i][3] = -Sq(c[i].r);
	}

	i2 = Max(0,1-i1);
	for (int i=0; i<3; i++) {
		if (i == i1) continue;
		for (int j=1; j<4; j++)
			a[i][j] = a[i][j] - a[i][0]*a[i1][j]/a[i1][0];
		if (Abs(a[i][1]) > Abs(a[i2][1])) i2=i;
	}

	i3 = 3-i1-i2;
	a[i3][2] =  a[i3][2]-a[i3][1] * a[i2][2]/a[i2][1];
	a[3][2]  = (a[i3][3]-a[i3][1] * a[i2][3]/a[i2][1]) / a[i3][2];
	a[3][1]  = (a[i2][3]-a[i2][2] * a[3][2]) / a[i2][1];
	a[3][0]  = (a[i1][3]-a[i1][2] * a[3][2]- a[i1][1]*a[3][1]) / a[i1][0];

	// set up coefficients for simplest applicable standard surface.
	// sphere if x² coefficient is 1.
	if (Abs(a[3][0]-1.)*PHUGE <= 1) {
		ks=3;
		// sphere at origin if linear term is zero.
		if (Abs(a[3][1])*PHUGE <= t) {
			P = Vector::O;
			xlen = ylen = zlen = -a[3][2];
			addQuad(Quad(Quad::Type::SPHERE_O, xlen));
		} else {
			// sphere, not at origin.
			double pos    = -.5*a[3][1];
			xlen = ylen = zlen = Sq(pos) - a[3][2];
			switch (type()) {
				case Type::X:
					P.set(pos,0,0);
					addQuad(Quad(Quad::Type::SPHERE, P, xlen));
					break;
				case Type::Y:
					P.set(0,pos,0);
					addQuad(Quad(Quad::Type::SPHERE, P, xlen));
					break;
				case Type::Z:
					P.set(0,0,pos);
					addQuad(Quad(Quad::Type::SPHERE, P, xlen));
					break;
				default:
					assert(false);
			}
			cout << "Adding: " << hs[ks] << " at ="
				<< " pos=" << P
				<< " R=" << xlen << endl;
			return;
		}
	}

//    1 p    2 px   3 py   4 pz   5 so   6 s    7 sx   8 sy   9 sz  10 c/x
//   11 c/y 12 c/z 13 cx  14 cy  15 cz  16 k/x 17 k/y 18 k/z 19 kx  20 ky
//   21 kz  22 sq  23 gq  24 tx  25 ty  26 tz  27 x   28 y   29 z   30 box
//   31 rpp 32 sph 33 rcc 34 rec 35 ell 36 trc 37 wed 38 arb 39 rhp
//
	// sq surface of some sort.
	double scf[10];
	for (int i=0; i<10; i++)	// Cxx/Cyy/Czz=1, all other C*=0
		scf[9-i] = i/7;
	for (int i=0; i<10; i++)
		printf("  %d:%g",i,scf[i]);
	cout << endl;

	// paraboloid if it has no x**2 term.  let constant term be zero.
	if (Abs(a[3][0])*PHUGE <= 1.) {
		ks=4;
//		m1c = 27; 27-26=1	// X
//		m1c = 28; 27-23=4	// Y
//		m1c = 29; 27-19=8	// Z
		// SQ
		scf[axis]   = 0;
		scf[axis+3] = .5*a[3][1];
		scf[axis+7] = -a[3][2]/a[3][1];
		switch (type()) {
			case Type::X:
				addQuad(0,0,0,	// 1:Cxx 2:Cyy 3:Czz
					0,0,0,	// 4:Cxy 5:Cxz 6:Cyz
					0,0,0,	// 7:Cx  8:Cy  9:Cz
					0);	// 10:C
				break;
			case Type::Y:
				break;
			case Type::Z:
				break;
			default:
				assert(0);
		}
		cout << "Adding: " << hs[ks] << endl;
		return;
	}

	// hyperboloid, ellipsoid, or cone.  let linear term be zero.
	scf[axis]   = a[3][0];
	scf[6]      = a[3][2] - .25* Sq(a[3][1]) / a[3][0];
	scf[axis+7] = -.5*a[3][1]/a[3][0];

	// two-sheet cone if constant term is zero.
	if (Abs(scf[6])*PHUGE <= t*t and a[3][0] <= 0) {
		scf[0] = scf[axis+7];
		scf[1] = -a[3][0];
		scf[2] = 0;
//		m1c = m1c-8;	// KX/KY/KZ
		ks = 8;
		cout << "Adding: " << hs[ks] << endl;
		for (int i=0; i<3; i++)
			printf("  %d:%g",i,scf[i]);
		cout << endl;
		return;
	}

	// hyperboloid or ellipsoid.
	ks = 5;
	if (scf[6]  > 0) ks = 6;
	if (a[3][0] > 0) ks = 7;

	// check that all points are on the same sheet of a hyperboloid.;
	if (scf[axis+7] < t2 and scf[axis+7] > t1 and ks == 6)
		printf("the points are on different sheets of a hyperboloid.\n");
//	m1c = 22;

	// print the shape of the translated surface.;
	if (ks>=0) cout << "Adding: " << hs[ks] << endl;
	for (int i=0; i<10; i++)
		printf("  %d:%g",i,scf[i]);
	cout << endl;
} // createQuads

/* createMesh */
void GXYZBody::createMesh()
{
	GBody::createMesh();
	if (nQ()==0) return;
	switch (Q(0).type()) {
		case Quad::Type::PLANE:
		case Quad::Type::PLANE_X:
		case Quad::Type::PLANE_Y:
		case Quad::Type::PLANE_Z:
			_mesh->createPlane(P, X, xlen, Y, ylen, Z, zlen);
			break;

		case Quad::Type::SPHERE:
		case Quad::Type::SPHERE_O:
		//case Quad::Type::SPHERE_X:
		//case Quad::Type::SPHERE_Y:
		//case Quad::Type::SPHERE_Z:
		case Quad::Type::ELLIPSOID:
			_mesh->createEllipsoid(P, X, xlen, Y, ylen, Z, zlen,
				GSphericalBase::N_SPHERE_LON, GSphericalBase::N_SPHERE_LAT);
			break;

		case Quad::Type::CYLINDER:
		case Quad::Type::CYLINDER_XO:
		case Quad::Type::CYLINDER_YO:
		case Quad::Type::CYLINDER_ZO:
		case Quad::Type::CYLINDER_X:
		case Quad::Type::CYLINDER_Y:
		case Quad::Type::CYLINDER_Z:
			_mesh->createEllipticalCone(P-Z*zlen, P+Z*zlen,
					X, xlen, xlen, Y, ylen, ylen,
					GCylinderBase::N_CYLINDER);
			break;

		//case Quad::Type::ELLIPTIC_CYLINDER:
		//case Quad::Type::ELLIPTIC_CYLINDER_X:
		//case Quad::Type::ELLIPTIC_CYLINDER_Y:
		//case Quad::Type::ELLIPTIC_CYLINDER_Z:
		//	break;

		//case Quad::Type::CONE:
		////case Quad::Type::CONE_XO:
		////case Quad::Type::CONE_YO:
		////case Quad::Type::CONE_ZO:
		//case Quad::Type::CONE_X:
		//case Quad::Type::CONE_Y:
		//case Quad::Type::CONE_Z:
		//	break;

		default:
			NOP;
	}
} // createMesh

/** @return bounding box of body */
void GXYZBody::_updateBBox()
{
	if (nQ())
		_bbox = Q(0).bbox();
	else
		_bbox.infinite();
} // _updateBBox

#ifdef USE_OBBOX
/** @return oriented bounding box of body */
void GXYZBody::_updateOBB(bool in)
{
	if (nQ())
		_obbox[(int)in] = OBBox(Q(0).obbox());
	else
		_obbox[(int)in].reset();
} // _updateOBB
#endif

///**
// * @param body	other body to check location against
// * @return location of this body with respect to another body
// *
// * FIXME TODO add bounding box limits!!!!
// */
//GBody::Location GXYZBody::_locationWrt(const GBody* body) const
//{
//} // _locationWrt
