/*
 * Copyright European Organization for Nuclear Research (CERN)
 * All rights reserved
 *
 * Please look at the supplied documentation for the user's
 * license
 *
 * Author:	Vasilis.Vlachoudis@cern.ch
 * Date:	01-Apr-2010
 */

#include <Python.h>

#include "viewerobject.h"
#include "geometryobject.h"

#define VERSION "3.3-1"

static PyModuleDef geoviewer_module = {
	PyModuleDef_HEAD_INIT,
	"geoviewer",
	"Geometry viewer extension module",
	-1,
	NULL, NULL, NULL, NULL, NULL
}; // geoviewer_module

PyMODINIT_FUNC PyInit_geoviewer(void)
{
	PyObject* m;

	//GeometryType.tp_new = PyType_GenericNew;
	if (PyType_Ready(&GeometryType)  < 0) return NULL;
	if (PyType_Ready(&ViewerType)    < 0) return NULL;

//	m = Py_InitModule3("geoviewer", NULL /*module methods*/,
//		"Geometry viewer extension type.");
	m = PyModule_Create(&geoviewer_module);
	if (m == NULL) return NULL;

	Py_INCREF(&GeometryType);
	Py_INCREF(&ViewerType);

	PyModule_AddStringConstant(m, "__author__",  "Vasilis Vlachoudis");
	PyModule_AddStringConstant(m, "__email__",   "Vasilis.Vlachoudis@cern.ch");
	PyModule_AddStringConstant(m, "__version__",  VERSION);

	PyModule_AddObject(m, "Geometry",  (PyObject *)&GeometryType);
	PyModule_AddObject(m, "Viewer",    (PyObject *)&ViewerType);

	return m;
} // initgeoviewer
