/*
 * Copyright European Organization for Nuclear Research (CERN)
 * All rights reserved
 *
 * Please look at the supplied documentation for the user's
 * license
 *
 * Author:	Vasilis.Vlachoudis@cern.ch
 * Date:	04-Feb-2010
 */

#include <Python.h>

#include <math.h>
#include <stdio.h>
#include <assert.h>
#include <stdlib.h>
#include <string.h>

#include "gmesh.h"
#include "pyutils.h"

#include "tool/timer.h"
#include "xdraw/xdraw.h"
#include "geometry/stl.h"
#include "viewerobject.h"

using std::cout;
using std::endl;

/* ================================= GMesh ================================== */
/** set parameter
 * @param key	value to set
 * @param value	python object to set value from
 * @param return true on error and set the python error
 */
PyObject* GMesh::config(const char* key, PyObject* value)
{
	if (!strcmp(key, "file")) {	// load an STL file
		if (value && PyUnicode_Check(value)) {
			STL stl;
			stl.read(PyUnicode_AsUTF8(value), mesh);
		}
	} else
	if (!strcmp(key, "bodies")) {	// return list of bodies found
		List<GBody*> list;
		mesh.fit(list,0.2);

		PyObject* ret = PyList_New(0);

		ListIterator<GBody*> iter(list);
		while (iter) {
			GBody* body = iter++;
			double what[30];
			int n = body->get(what);
			PyObject* obj = PyList_New(1+n);
			PyList_SetItem(obj, 0, PyUnicode_FromString(body->typeStr()));
			for (int i=0; i<n; i++)
				PyList_SetItem(obj, i+1, PyFloat_FromDouble(what[i]));
			PyList_Append(ret, obj);
			delete body;
		}
		return ret;
	} else
	if (!strcmp(key, "print")) {	// load an STL file
		cout << mesh << endl;
	} else
		return GObject::config(key, value);

	if (PyErr_Occurred()) return NULL;
	Py_RETURN_NONE;
} // config

/** draw */
void GMesh::draw(ViewerObject* self, Drawable drawable)
{
	GObject::draw(self, drawable);

	// FIXME VERY PRIMITIVE
	// DOUBLE draws lines
	//mesh.clearProcessedFlag();
	for (size_t i=0; i<mesh.nedges(); i++) {
		const Edge* edge = mesh.edge(i);
		if (edge->show) {
			Vector A = self->kernel->view.xyz2uvw(edge->A() + P);
			Vector B = self->kernel->view.xyz2uvw(edge->B() + P);
			self->xdraw.line3D(drawable, self->kernel->view, A, B);
		}
		//f->processed(true);
	}
} // draw
