/*
 * Copyright European Organization for Nuclear Research (CERN)
 * All rights reserved
 *
 * Please look at the supplied documentation for the user's
 * license
 *
 * Author:	Vasilis.Vlachoudis@cern.ch
 * Date:	04-Feb-2010
 */

#ifndef __PYUTILS_H
#define __PYUTILS_H

#include <Python.h>
#include "math/matrix3.h"
#include "math/matrix4.h"

#if PY_MAJOR_VERSION >= 3
#	define IS_PY3K
#endif

// Definitions for older pythons
#ifndef Py_RETURN_NONE
#	define Py_RETURN_NONE  return Py_INCREF(Py_None), Py_None
#	define Py_RETURN_TRUE  return Py_INCREF(Py_True), Py_True
#	define Py_RETURN_FALSE return Py_INCREF(Py_False), Py_False
#endif
#ifndef Py_TYPE
#	define Py_TYPE(ob)	(((PyObject*)(ob))->ob_type)
#endif

// function prototypes
bool	Py_GetBool(PyObject* obj);
int	Py_GetInt(PyObject* obj);
double	Py_GetFloat(PyObject* obj);
bool	Py_GetUV(PyObject* obj, double* u, double* v);

bool	Py_IsVector(PyObject* obj);
Vector	Py_GetVector(PyObject* obj);
Point	Py_GetPoint(PyObject* obj);
inline	PyObject* Py_Vector(const Vector& v)
		{ return Py_BuildValue("ddd", v.x, v.y, v.z); }

bool	PyList_AsMatrix3(PyObject* obj, Matrix3& matrix);
bool	PyList_AsMatrix4(PyObject* obj, Matrix4& matrix);
PyObject* PyList_FromMatrix3(const Matrix3& matrix);
PyObject* PyList_FromMatrix3(const Matrix4& matrix);
PyObject* PyList_FromMatrix4(const Matrix4& matrix);
bool	Py_Check4Pattern(PyObject* obj);

#endif
